/*
 * AutoDAO - Generic DAO on steroids implementation for Java.
 *
 * Copyright 2008-2012  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.autodao.impl;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.xml.AbstractSingleBeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

/**
 * Processes <code>dao:XXX</code> elements and builds Spring bean.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
final class AutoDAOBeanDefinitionParser extends AbstractSingleBeanDefinitionParser {

  @NotNull
  private final Class<? extends AbstractDaoFactoryBean> beanClass;
  @NotNull
  private final String customParamName;
  @NotNull
  private final String customParamFallback;

  AutoDAOBeanDefinitionParser(@NotNull Class<? extends AbstractDaoFactoryBean> beanClass,
                              @NotNull String customParamName,
                              @NotNull String customParamFallback) {
    this.beanClass = beanClass;
    this.customParamName = customParamName;
    this.customParamFallback = customParamFallback;
  }

  /** {@inheritDoc} */
  @Override
  protected boolean shouldGenerateIdAsFallback() {
    return true;
  }

  /** {@inheritDoc} */
  @Override
  protected void doParse(final Element element, final ParserContext context, final BeanDefinitionBuilder builder) {
    applyDaoParams(builder, element, element.getAttribute("interface"), element.getAttribute("entity"), customParamName, customParamFallback);
  }

  @Override
  protected Class getBeanClass(final Element element) {
    return beanClass;
  }

  static void applyDaoParams(@NotNull final BeanDefinitionBuilder builder,
                             @NotNull final Element element,
                             @NotNull final String daoInterfaceName,
                             @Nullable final String entityName,
                             @NotNull final String customParamName,
                             @NotNull final String customParamFallback) {
    final String transactionManager = element.hasAttribute("transaction-manager")
        ? element.getAttribute("transaction-manager")
        : "transactionManager";

    final String customParam = element.hasAttribute(customParamName)
        ? element.getAttribute(customParamName)
        : customParamFallback;

    builder
        .addConstructorArgValue(daoInterfaceName)
        .addConstructorArgValue(entityName)
        .addConstructorArgReference(transactionManager)
        .addConstructorArgReference(customParam);
  }
}
